/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

/*
 * Concepts:
 * !Shared memory (RChunk)!
 * !Disk space notification!
 * !Debug call stack!
 */

#include "break.h"
#include "app_context.h"
#include "app_context_impl.h"
#include "app_context_fileutil.h"
#include "settings.h"

#include <f32file.h>
#include <charconv.h>
#include <etel.h>
#include <etelbgsm.h>
#include <saclient.h>
#include <w32std.h>
#include "list.h"
#include <stdlib.h> // for CloseSTDLIB()

#ifndef __S80__
#include <aknglobalnote.h>
#endif

#include <coemain.h>
#include <flogger.h>
#include <bautils.h>
#include "context_uids.h"
#ifdef __WINS__
#include <apgcli.h>
#include <apgtask.h>
#endif
#include "raii.h"
#include "symbian_auto_ptr.h"
#include "raii_apgcli.h"
#include "raii_f32file.h"
#include "reporting.h"
#include "pausable.h"
#include "errorinfo.h"
#include "callstack.h"
#include <avkon.rsg>

//#define DEBUG_CALLSTACK 1

class CApp_contextImpl : public CApp_context, public MDiskSpace, public MReporting,
	public MErrorInfoManager, public MCallStack {
public:
	virtual ~CApp_contextImpl();
	virtual RFs&	Fs();
	virtual RSystemAgent&	SysAgent();
	virtual RTelServer&	TelServer();
	virtual RBasicGsmPhone&	Phone();
	virtual CCnvCharacterSetConverter* CC();
	virtual bool NoSpaceLeft();
	virtual void SetFileLog(MPausable* log);
	virtual const TDesC& DataDir();
	virtual const TDesC& AppDir();
	virtual void SetDataDir(const TDesC& Dir, bool UseMMC);
	virtual void SetAppDir(const TDesC& Dir);
	virtual MSettings& Settings(); 
	virtual void SetActiveErrorReporter(MActiveErrorReporter* Reporter);
	virtual MActiveErrorReporter* GetActiveErrorReporter();
	virtual void SetSettings(MSettings* Settings);  // takes ownership
	virtual RWsSession& Ws();
	virtual void ReportActiveError(const TDesC& Source,
		const TDesC& Reason, TInt Code);
	virtual const char* CallStack();
	virtual TInt PushCallStack(const TDesC8& Name);
	virtual TInt PushCallStack(const TDesC8& Class, const TDesC8& Func);
	virtual TPtr8 PopCallStack();
	virtual void ResetCallStack();
	virtual HBufC* GetFormattedCallStack(const TDesC& Prefix);
	virtual void IterateStack(MCallStackVisitor& aVisitor, const char* aStack=0);
	virtual void ShowGlobalNote(TInt notetype, const TDesC& msg);
	virtual TPtrC8 GetCurrentClass();

	virtual MErrorInfo* GetLastErrorInfo();
	virtual MErrorInfo* GetLastErrorInfo(TErrorCode aCode);
	virtual void ResetLastErrorInfo();
	virtual void SetBBSession(CBBSession* BBSession);
	virtual CBBSession* BBSession();
	virtual void SetBBDataFactory(MBBDataFactory* aFactory);
	virtual MBBDataFactory* BBDataFactory();
	virtual MReporting& Reporting();
	virtual MErrorInfoManager& ErrorInfoMgr();
	virtual MCallStack& CallStackMgr();

	virtual void SetDebugLog(const TDesC& aDir, const TDesC& aFile);
	virtual void DebugLog(const TDesC& aMessage);
	virtual void DebugLog(const TDesC& aMessage, TInt aCode);
        virtual void UserErrorLog(const TDesC& aMessage);
        virtual void UserErrorLog(const TDesC& aMessage, TInt aCode);
        virtual void UserErrorLog(const TDesC& aMessage, const TDesC& aMessagePart);
        virtual void UserErrorLog(const TDesC& aMessage,
                const TDesC& aMessagePart, const TDesC& aMessagePart2);
	void DebugLogL(const TDesC& aMessage);

//FIXME
	void SetHBufC(HBufC*& aBuf, const TStringArg& aString, TInt& aCount, TInt& aPos) {
		TInt len=aString.Length();
		if (len==0) aCount=0;
		else aCount=1;

		aPos=0;
		if (!aBuf) {
			aBuf=HBufC::New(aString.Length());
		} else {
			aBuf->Des().Zero();
			if (aBuf->Des().MaxLength() < len) {
				HBufC* tmp=HBufC::New(len);
				if (tmp) {
					delete aBuf;
					aBuf=tmp;
				}
			}
		}
		if (aBuf) {
			TPtr b=aBuf->Des();
			aString.AppendToString(b);
		}
	}
	void AppendHBufC(HBufC*& aBuf, const TStringArg& aString, TInt& aCount, TInt& aPos) {
		TInt len=aString.Length();
		if (!aBuf) {
			aBuf=HBufC::New(len);
		} else {
			if (aBuf->Des().MaxLength()-aBuf->Des().Length() < len) {
				CC_TRAPD(err, aBuf=aBuf->ReAllocL(aBuf->Des().Length()+len));
			}
		}
		if (aBuf) {
			TInt formatpos=KErrNotFound;
			TBuf<10> format=_L("%");
			format.AppendNum(aCount++);
			if (aPos>=0) {
				formatpos=aBuf->Des().Mid(aPos).Find(format);
			}
			if ( formatpos==KErrNotFound ) {
				aPos=0;
				TPtr b=aBuf->Des();
				aString.AppendToString(b);
			} else {
				formatpos+=aPos;
				aPos=formatpos+format.Length();
				HBufC* rest=0;
				CC_TRAPD(err, rest=(*aBuf).Mid(aPos).AllocL());
				if (rest) {
					auto_ptr<HBufC> resta(rest);
					aBuf->Des().SetLength(formatpos);
					TPtr b=aBuf->Des();
					aString.AppendToString(b);
					aPos=(*aBuf).Length();
					aBuf->Des().Append(*rest);
				} else {
					// skip insert if out of memory
					format.Fill('.');
				}
			}
		} else {
			++aCount;
		}
	}
	HBufC	*iUserMsg, *iTechMsg;
	HBufC	*iReportingMsg;
	TErrorCode	iErrorCode;
	TErrorType	iErrorType;
	TSeverity	iSeverity;
	MErrorInfo	*iLastErrorInfo;
	TInt		iUserMsgCount, iTechMsgCount;
	TInt		iUserMsgPos, iTechMsgPos;
	virtual void StartErrorWithInfo(TErrorCode aErrorCode,
		const TStringArg& aUserMessage, const TStringArg& aTechMessage,
		TErrorType aErrorType, TSeverity aSeverity) { 

		iErrorCode=aErrorCode;
		iErrorType=aErrorType;
		iSeverity=aSeverity;
		SetHBufC(iUserMsg, aUserMessage, iUserMsgCount, iUserMsgPos);
		SetHBufC(iTechMsg, aTechMessage, iTechMsgCount, iTechMsgPos);
	}
	virtual MErrorInfoManager& UserMsg(const TStringArg& aArg) { 
		AppendHBufC(iUserMsg, aArg, iUserMsgCount, iUserMsgPos); 
		return *this; }
	virtual MErrorInfoManager& TechMsg(const TStringArg& aArg) { 
		AppendHBufC(iTechMsg, aArg, iTechMsgCount, iTechMsgPos); 
		return *this; 
	}
	virtual MErrorInfoManager& ErrorCode(TErrorCode aErrorCode) { 
		iErrorCode=aErrorCode;
		return *this; 
	}
	virtual MErrorInfoManager& Severity(TSeverity aSeverity) { 
		iSeverity=aSeverity;
		return *this; 
	}
	virtual MErrorInfoManager& ErrorType(TErrorType aErrorType) { 
		iErrorType=aErrorType;
		return *this; 
	}
	virtual void Raise() {
		HBufC* callstack=GetFormattedCallStack(KNullDesC);
		MErrorInfo* tmp=0;
		CC_TRAPD(err, tmp=CErrorInfo::NewL(iErrorCode, iUserMsg, iTechMsg, callstack,
			iErrorType, iSeverity, iLastErrorInfo));
		if (tmp) {
			iLastErrorInfo=tmp;
			iUserMsg=0;
			iTechMsg=0;
		} else {
			delete callstack;
		}
		User::Leave(iErrorCode.iCode);
	}
	virtual void Log() {
	}

	CList<MBusyIndicator*>	*iBusyIndicators; TBool iIsBusy;
	virtual TBool AddBusyIndicator(MBusyIndicator *aIndicator) 
	{
		iBusyIndicators->AppendL(aIndicator);
		return iIsBusy;
	}
	virtual void RemoveBusyIndicator(MBusyIndicator *aIndicator) 
	{
		CList<MBusyIndicator*>::Node *n=iBusyIndicators->iFirst;
		while(n) {
			if (n->Item==aIndicator) {
				iBusyIndicators->DeleteNode(n, ETrue);
				return;
			}
			n=n->Next;
		}
	}
	virtual TBool ShowBusy(TBool aIsBusy)
	{
		TBool tmp=iIsBusy;
		CList<MBusyIndicator*>::Node* n=iBusyIndicators->iFirst;
		while(n) {
			n->Item->ShowBusy(aIsBusy);
			n=n->Next;
		}
		iIsBusy=aIsBusy;
		return tmp;
	}
	TBuf<128> iState;
	virtual void SetState(const TDesC& aState) {
		if (aState.Length()<=128) {
			iState=aState;
		} else {
			iState=aState.Left(128);
		}
	}
	virtual void TakeOwnershipL(CBase* aObject) {
		iOwnedObjects[iOwnedObjects.Count()-1]=aObject;
		User::LeaveIfError(iOwnedObjects.Append(0));
	}
	void(*iRunOnShutdown)(void);
	virtual void RunOnShutdown( void(*function)(void) ) {
		iRunOnShutdown=function;
	}


private:
	CApp_contextImpl();
	void ConstructL(bool aFsOnly, const TDesC& Name);
	virtual void DiskSpaceThreshold(TInt aDrive);

	RFs	iFs;
	RSystemAgent	iSysAgent; bool hasSysAgent;
	RTelServer	iTelServer; bool hasTelServer;
#ifndef __WINS__
	RBasicGsmPhone	iPhone; bool hasPhone;
#endif
	CCnvCharacterSetConverter* cc; 
	TInt64		iDiskSpaceLimit;
	TInt		iDrive[2];
	bool		iSpaceLeft;
#ifndef __S80__
	CAknGlobalNote *globalNote;
#endif
	MPausable	*iFileLog;
	MSettings	*iSettings;
	MActiveErrorReporter	*iReporter;
	TFileName	iDataDir;
	TFileName	iAppDir;
	bool		iFsOnly;
	bool		iUseMMC;

	RChunk		iCallStackChunk; int iCallStackSize; bool has_chunk;
	char		*iCallStackZero, *iCallStackCurrent;

	CDiskSpaceNotifier *iDiskSpaceNotifier[2];

	CBBSession	*iBBSession;

	TBuf<50>	iDebugDir, iDebugFile;

	RFileLogger iDebugLog; TBool iDebugLogIsOpen;
	TBool		iDebugCallStack;

	MBBDataFactory*	iBBDataFactory;

	RPointerArray<CBase>	iOwnedObjects;

	friend CApp_context;
};

class CDiskSpaceNotifier: public CCheckedActive
{
public:
	static CDiskSpaceNotifier* NewL(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold);
	~CDiskSpaceNotifier();
private:
	CDiskSpaceNotifier(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold);
	void ConstructL();
	void CheckedRunL();
	void DoCancel();

	MDiskSpace* iNotifier;
	RFs& iFs;
	TInt iDrive;
	TInt64 iThreshold;
};

EXPORT_C TTime GetTime() { 
	TTime t; t.UniversalTime(); t+=TTimeIntervalHours(3);
	return t; 
}

EXPORT_C TTime ContextToLocal(TTime aTime) {
	TLocale locale;
	TTimeIntervalSeconds universalTimeOffset(locale.UniversalTimeOffset());
	TTime ret=aTime;
	ret-=TTimeIntervalHours(3);
	ret+=universalTimeOffset;
	return ret;
}


EXPORT_C void print_stacktrace(RFs& fs, MApp_context* context, const TDesC& cat, TInt error)
{
	if (!context) return;
	RFile f;
	if (f.Open(fs, _L("C:\\crash.txt"), 
		EFileShareAny | EFileStreamText | EFileWrite)!=KErrNone) {
		if(f.Create(fs, _L("C:\\crash.txt"), 
			EFileShareAny | EFileStreamText | EFileWrite)!=KErrNone) {
			return;
		}
	}
	CleanupClosePushL(f);

	TInt pos=0;
	f.Seek(ESeekEnd, pos);

	TBuf<200> msg;
	msg.Format(_L("context_log crashed with error %d\ncategory "), error);
	TPtrC8 msg2( (TText8*)msg.Ptr(), msg.Length()*2);
	f.Write(msg2);
	TPtrC8 msg3( (TText8*)cat.Ptr(), cat.Length()*2);
	f.Write(msg3);
	f.Write(_L8("\ntrace:\n"));

	TPtr8 n=context->CallStackMgr().PopCallStack();
	while (n.Ptr()) {
		msg.Zero(); msg.Copy(n);
		f.Write(n);
		n=context->CallStackMgr().PopCallStack();
	}
	CleanupStack::PopAndDestroy();
}

EXPORT_C void exceptionhandler(TExcType t)
{
	MApp_context* c;
	c=MApp_context::Static();
	if (c) {
		print_stacktrace(c->Fs(), c, _L("Exception"), t);
	}
	User::Panic(_L("EXCEPTION"), t);
}

MApp_context::~MApp_context()
{
}

CApp_contextImpl::CApp_contextImpl()
{
}

void CApp_contextImpl::ConstructL(bool aFsOnly, const TDesC& Name)
{
	RFileLogger iLog;
	TInt log_err=iLog.Connect();
	if (log_err==KErrNone) {
		iLog.CreateLog(_L("Context"),_L("CApp_contextImpl.txt"),EFileLoggingModeAppend);
		iLog.Write(_L("constructing"));
	}

	iFsOnly=aFsOnly;

	iDiskSpaceLimit=500*1024; // 500k
	iDrive[0]=2; // C:
	iFileLog=0;

	User::LeaveIfError(iFs.Connect());
	iSpaceLeft=true;
	iBusyIndicators=CList<MBusyIndicator*>::NewL();

	if (Name.Length() && (iCallStackChunk.OpenGlobal(Name, EFalse)==KErrNone || 
		iCallStackChunk.CreateGlobal(Name, 4096, 4096)==KErrNone)) {
		has_chunk=true;
		iCallStackZero=iCallStackCurrent=(char*)iCallStackChunk.Base() + 4;
		iCallStackSize=iCallStackChunk.Size();
	} else {
		has_chunk=false;
		iCallStackZero=new char[4096];
		if (iCallStackZero) iCallStackZero += 4;
		iCallStackCurrent=iCallStackZero;
		iCallStackSize=4096;
	}

	if (!aFsOnly) {
#ifndef __S80__
		globalNote=CAknGlobalNote::NewL();
#endif
	}
	

	User::LeaveIfError(iOwnedObjects.Append(0));
	// only the first app context in a thread gets put
	// in the Tls, so that we can use an instance to look
	// at another thread's call stack
	if (Dll::Tls()==0) Dll::SetTls(this);

	if (log_err==KErrNone) {
		iLog.Write(_L("constructed"));

		iLog.CloseLog();
		iLog.Close();
	}
}

void CApp_contextImpl::SetBBSession(CBBSession* BBSession)
{
	iBBSession=BBSession;
}

CBBSession* CApp_contextImpl::BBSession()
{
	return iBBSession;
}

void CApp_contextImpl::SetBBDataFactory(MBBDataFactory* aFactory)
{
	iBBDataFactory=aFactory;
}

MBBDataFactory* CApp_contextImpl::BBDataFactory()
{
	return iBBDataFactory;
}

void CApp_contextImpl::SetDebugLog(const TDesC& aDir, const TDesC& aFile)
{
	iDebugDir=aDir;
	iDebugFile=aFile;
	if (iDebugLogIsOpen) {
		iDebugLog.Close();
		iDebugLogIsOpen=EFalse;
	}
	if (iDebugLog.Connect()==KErrNone) {
		iDebugLogIsOpen=ETrue;
		iDebugLog.CreateLog(iDebugDir, iDebugFile, EFileLoggingModeAppend);
		iDebugLog.Write(_L("started log"));
	}
}

void CApp_contextImpl::DebugLogL(const TDesC& msg)
{
	TInt i=0;
	while (i<msg.Length()) {
		iDebugLog.Write(msg.Mid(i));
		i+=128;
	}	
}

void CApp_contextImpl::DebugLog(const TDesC& aMessage)
{
#ifdef __WINS__
	RDebug::Print(aMessage.Left(128));
#endif
	if (!iDebugLogIsOpen) return;
	CC_TRAPD(err, DebugLogL(aMessage));
}

void CApp_contextImpl::DebugLog(const TDesC& aMessage, TInt aCode)
{
#ifdef __WINS__
	TBuf<12> num; num.AppendNum(aCode);
	RDebug::Print(aMessage.Left(128));
	RDebug::Print(num);
#endif
	if (!iDebugLogIsOpen) return;
#ifndef __WINS__
	TBuf<12> num; num.AppendNum(aCode);
#endif
	CC_TRAPD(err, DebugLogL(aMessage));
	CC_TRAP(err, DebugLogL(num));
}

void CApp_contextImpl::UserErrorLog(const TDesC& aMessage, TInt aCode)
{
	if (iReporter) {
		TInt c, p;
		SetHBufC(iReportingMsg, aMessage, c, p);
		AppendHBufC(iReportingMsg, aCode, c, p);

		if (iReportingMsg)
			iReporter->LogFormatted(*iReportingMsg);
	}
	DebugLog(aMessage, aCode);
}
void CApp_contextImpl::UserErrorLog(const TDesC& aMessage)
{
	if (iReporter) {
		iReporter->LogFormatted(aMessage);
	}
	DebugLog(aMessage);
}
void CApp_contextImpl::UserErrorLog(const TDesC& aMessage, const TDesC& aMessagePart)
{
	if (iReporter) {
		TInt c, p;
		SetHBufC(iReportingMsg, aMessage, c, p);
		AppendHBufC(iReportingMsg, aMessagePart, c, p);

		if (iReportingMsg)
			iReporter->LogFormatted(*iReportingMsg);
	}
	DebugLog(aMessage);
	DebugLog(aMessagePart);
}
void CApp_contextImpl::UserErrorLog(const TDesC& aMessage,
        const TDesC& aMessagePart, const TDesC& aMessagePart2)
{
	if (iReporter) {
		TInt c, p;
		SetHBufC(iReportingMsg, aMessage, c, p);
		AppendHBufC(iReportingMsg, aMessagePart, c, p);
		AppendHBufC(iReportingMsg, aMessagePart2, c, p);

		if (iReportingMsg)
			iReporter->LogFormatted(*iReportingMsg);
	}
	DebugLog(aMessage);
	DebugLog(aMessagePart);
	DebugLog(aMessagePart2);
}

CApp_contextImpl::~CApp_contextImpl()
{
	if (Dll::Tls()==this) Dll::SetTls(0);

	iOwnedObjects.ResetAndDestroy();

	delete iBusyIndicators;

	delete iDiskSpaceNotifier[0];
	delete iDiskSpaceNotifier[1];

	delete iSettings;
#ifndef __S80__
	delete globalNote;
#endif

	if (!has_chunk) {
		iCallStackZero-=4;
		delete iCallStackZero;
	} else {
		iCallStackZero-=4;
		*(TInt*)iCallStackZero=0;
		iCallStackChunk.Close();
	}
	iCallStackZero=iCallStackCurrent=0;

	if (!iFsOnly) {
	}

#if !defined(__WINS__) && !defined(__S60V2__)
	if (hasPhone) iPhone.Close();
#  if !defined(__WINS__)
	_LIT(KGsmModuleName, "phonetsy.tsy");
#  else
	_LIT(KGsmModuleName, "mm.tsy");
#endif
	if (hasTelServer) {
		iTelServer.UnloadPhoneModule( KGsmModuleName );
		iTelServer.Close();
	}
#endif
	if (hasSysAgent) iSysAgent.Close();

	delete cc;
	iFs.Close();

	delete iLastErrorInfo;
	iLastErrorInfo=0;

	RFileLogger log;
	if (log.Connect()==KErrNone) {
		log.CreateLog(_L("Context"),_L("CApp_contextImpl.txt"),EFileLoggingModeAppend);
		log.Write(iState);
		log.Write(_L("deleted"));

		log.CloseLog();
		log.Close();
	}

	if (iDebugLogIsOpen) {
		iDebugLog.CloseLog();
		iDebugLog.Close();
	}

	delete iUserMsg; delete iTechMsg;
	delete iReportingMsg;
	//CloseSTDLIB();
	if (iRunOnShutdown) { (*iRunOnShutdown)(); }
}

EXPORT_C CApp_context* CApp_context::NewL(bool aFsOnly, const TDesC& Name)
{
	CApp_contextImpl* ret;
	ret=new (ELeave) CApp_contextImpl();
	CleanupStack::PushL(ret);
	ret->ConstructL(aFsOnly, Name);
	CleanupStack::Pop();
	return ret;
}

CApp_context::~CApp_context()
{
}

RFs& CApp_contextImpl::Fs()
{
	return iFs;
}

RSystemAgent& CApp_contextImpl::SysAgent()
{
	if (!hasSysAgent) {
		User::LeaveIfError(iSysAgent.Connect());
		hasSysAgent=true;
	}

	return iSysAgent;
}

RTelServer& CApp_contextImpl::TelServer()
{
#if !defined(__WINS__) || defined(__S60V2__)
	if (!hasTelServer) {
		User::LeaveIfError(iTelServer.Connect());
#  ifndef __WINS__
		_LIT(KGsmModuleName, "phonetsy.tsy");
#  else
		_LIT(KGsmModuleName, "mm.tsy");
#  endif
		TInt err;
		err=iTelServer.LoadPhoneModule( KGsmModuleName );
		if (err==KErrNone) {
			hasTelServer=true;
		} else {
			iTelServer.Close();
			User::Leave(err);
		}
	}
#endif
	return iTelServer;

}

RBasicGsmPhone& CApp_contextImpl::Phone()
{
#ifndef __WINS__
#if !defined(__WINS__) || defined(__S60V2__)
	if (!hasPhone) {
		RTelServer::TPhoneInfo info;
		User::LeaveIfError( TelServer().GetPhoneInfo( 0, info ) );
		User::LeaveIfError( iPhone.Open( iTelServer, info.iName ) );
		hasPhone=true;
	}
#endif

	return iPhone;
#else
	return *(RBasicGsmPhone*)0;
#endif
}

CCnvCharacterSetConverter* CApp_contextImpl::CC()
{
	if (!cc) {
		cc=CCnvCharacterSetConverter::NewL();
		cc->PrepareToConvertToOrFromL(KCharacterSetIdentifierIso88591, iFs);
	}
	return cc;
}

void CApp_contextImpl::SetFileLog(MPausable* log)
{
	iFileLog=log;
	if (iFileLog && !iSpaceLeft) iFileLog->pause();
}

void CApp_contextImpl::DiskSpaceThreshold(TInt /*aDrive*/)
{
	int v;
	iSpaceLeft=true;
	TBuf<100> msg;
#ifndef __S80__
	TAknGlobalNoteType notetype;
#endif
	for(v=0; v<2; v++) {
		if (iDrive[v] < 0) continue;

		TVolumeInfo info;
		if (iFs.Volume(info, iDrive[v])==KErrNone) {
			if (info.iFree<=iDiskSpaceLimit) {
				iSpaceLeft=false;
			}
		}
	}
	if (!iSpaceLeft) {
		if (iFileLog) iFileLog->pause();
		msg.Append(_L("WARNING! No space left for logs. Please delete some pictures and other media."));
#ifndef __S80__
		notetype=EAknGlobalErrorNote;
#endif
	} else {
		if (iFileLog) iFileLog->unpause();
		msg.Append(_L("Logging continues."));	
#ifndef __S80__
		notetype=EAknGlobalInformationNote;
#endif
	}
#ifndef __S80__
	if (globalNote) {
		CC_TRAPD(err, globalNote->ShowNoteL(notetype, msg));
	}
#endif
}

void CApp_contextImpl::ShowGlobalNote(TInt ntype, const TDesC& msg)
{
#ifndef __S80__
	TAknGlobalNoteType notetype=(TAknGlobalNoteType)ntype;
	if (!globalNote) {
		TRAPD(err, globalNote=CAknGlobalNote::NewL());
	}
	if (globalNote) {
		globalNote->SetSoftkeys(R_AVKON_SOFTKEYS_OK_EMPTY);
		CC_TRAPD(err, globalNote->ShowNoteL(notetype, msg));
	}
#endif
}
void CDiskSpaceNotifier::ConstructL()
{
	CActiveScheduler::Add(this);
	iStatus=KRequestPending;
	iFs.NotifyDiskSpace(iThreshold, iDrive, iStatus);
	SetActive();
}

CDiskSpaceNotifier* CDiskSpaceNotifier::NewL(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold)
{
	auto_ptr<CDiskSpaceNotifier> ret(new (ELeave) CDiskSpaceNotifier(fs, aDrive, aNotifier, aThreshold));
	ret->ConstructL();
	return ret.release();
}

CDiskSpaceNotifier::~CDiskSpaceNotifier()
{
	Cancel();
}

CDiskSpaceNotifier::CDiskSpaceNotifier(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold) : 
	CCheckedActive(EPriorityNormal, _L("CDiskSpaceNotifier")), iNotifier(aNotifier), iFs(fs), iDrive(aDrive),
	iThreshold(aThreshold)
{
}

void CDiskSpaceNotifier::CheckedRunL()
{
	if (iStatus==KErrNone) {
		iNotifier->DiskSpaceThreshold(iDrive);
	}
	iStatus=KRequestPending;
	iFs.NotifyDiskSpace(iThreshold, iDrive, iStatus);
	SetActive();
}

void CDiskSpaceNotifier::DoCancel()
{
	iFs.NotifyDiskSpaceCancel();
}

bool CApp_contextImpl::NoSpaceLeft()
{
	if (iFsOnly) return false;

	if (!iDiskSpaceNotifier[0]) {
		iDrive[0]=2;
		iDiskSpaceNotifier[0]=CDiskSpaceNotifier::NewL(iFs, 2, this, iDiskSpaceLimit);

#ifndef __WINS__
		{
		iDrive[1]=-1;
		if (iUseMMC && HasMMC(iFs)) {
			// memory card
			iDrive[1]=4;
			iDiskSpaceNotifier[1]=CDiskSpaceNotifier::NewL(iFs, 4, this, iDiskSpaceLimit);
		}
		}
#endif

		int v;
		for(v=0; v<2; v++) {
			if (iDrive[v] < 0) continue;
			TVolumeInfo volinfo;
			if (iFs.Volume(volinfo, iDrive[v])==KErrNone) {
				if (volinfo.iFree<=iDiskSpaceLimit) {
					iSpaceLeft=false;
				}
			}
		}
	}

	return !iSpaceLeft;
}

const TDesC& CApp_contextImpl::DataDir()
{
	return iDataDir;
}

void CApp_contextImpl::SetDataDir(const TDesC& Dir, bool UseMMC)
{
	iUseMMC=UseMMC;
	BaflUtils::EnsurePathExistsL(iFs, Dir);
	if (UseMMC && HasMMC(iFs)) {
		// memory card

		TFileName f=Dir;
		f.Replace(0, 1, _L("E"));
		CC_TRAPD(err, BaflUtils::EnsurePathExistsL(iFs, f));
	}
	iDataDir=Dir;
}

const TDesC& CApp_contextImpl::AppDir()
{
	return iAppDir;
}

void CApp_contextImpl::SetAppDir(const TDesC& Dir)
{
	iAppDir=Dir;
}

void CApp_contextImpl::SetSettings(MSettings* Settings)
{
	iSettings=Settings;
}

MSettings& CApp_contextImpl::Settings()
{
	return *iSettings;
}

RWsSession& CApp_contextImpl::Ws()
{
	return CCoeEnv::Static()->WsSession();
}

const char* CApp_contextImpl::CallStack()
{
	return iCallStackCurrent;
}

void CApp_contextImpl::SetActiveErrorReporter(MActiveErrorReporter* Reporter)
{
	iReporter=Reporter;
}

MActiveErrorReporter* CApp_contextImpl::GetActiveErrorReporter()
{
	return iReporter;
}

void CApp_contextImpl::ReportActiveError(const TDesC& Source,
	const TDesC& Reason, TInt Code)
{
	if (iReporter) {
		iReporter->ReportError(Source, Reason, Code);
	} else {
		TFileName fn=_L("c:\\context_error.txt");
		{
			auto_ptr<HBufC> stack(GetFormattedCallStack(_L("Call stack")));
			RAFile f; f.ReplaceLA(Fs(), fn, EFileWrite);
			f.Write(_L8("\xff\xfe"));

			TBuf<40> m=_L("Unhandled error: ");
			m.AppendNum(Code); m.Append(_L(" "));
			TPtrC8 p;
			p.Set( (TUint8*)m.Ptr(), m.Size());
			f.Write(p);
			p.Set( (TUint8*)Reason.Ptr(), Reason.Size());
			f.Write(p);
			m=_L("\nin ");
			p.Set( (TUint8*)m.Ptr(), m.Size());
			f.Write(p);

			p.Set( (TUint8*)Source.Ptr(), Source.Size());
			f.Write(p);
			if (stack.get()) {
				TPtrC stackp=stack->Des();
				m=_L("\n");
				p.Set( (TUint8*)m.Ptr(), m.Size());
				f.Write(p);
				p.Set( (TUint8*)stackp.Ptr(), stackp.Size());
				f.Write(p);
			}
		}
		RAApaLsSession session; session.ConnectLA();
		TUid uid;
		TDataType dataType;
		session.AppForDocument(fn, uid, dataType);
		TThreadId threadId;
		User::LeaveIfError(session.StartDocument(fn, dataType, threadId));
		//FIXME: this should stop the active scheduler, to-be-tested
		User::Leave(Code);
	}
}

EXPORT_C MApp_context* MApp_context::Static()
{
	return (MApp_context*)Dll::Tls();
}

EXPORT_C const TDesC& MContextBase::DataDir()
{
	return iContext.DataDir();
}

EXPORT_C const TDesC& MContextBase::AppDir()
{
	return iContext.AppDir();
}

EXPORT_C MSettings& MContextBase::Settings()
{
	return iContext.Settings();
}


EXPORT_C bool MContextBase::NoSpaceLeft()
{
	return iContext.NoSpaceLeft();
}

EXPORT_C MContextBase::MContextBase() : iContext(*(MApp_context::Static()))
{
}

EXPORT_C MContextBase::MContextBase(MApp_context& Context) : iContext(Context)
{
}


EXPORT_C MContextBase::~MContextBase()
{
}

EXPORT_C RFs& MContextBase::Fs()
{
	return iContext.Fs();
}

EXPORT_C RSystemAgent& MContextBase::SysAgent()
{
	return iContext.SysAgent();
}

EXPORT_C RTelServer& MContextBase::TelServer()
{
	return iContext.TelServer();
}

EXPORT_C RBasicGsmPhone& MContextBase::Phone()
{
	return iContext.Phone();
}

EXPORT_C MApp_context& MContextBase::AppContext()
{
	return iContext;
}
EXPORT_C MApp_context_access& MContextBase::AppContextAccess()
{
	return iContext;
}

EXPORT_C CCnvCharacterSetConverter* MContextBase::CC()
{
	return iContext.CC();
}

EXPORT_C RWsSession& MContextBase::Ws()
{
	return iContext.Ws();
}

EXPORT_C void MContextBase::ReportActiveError(const TDesC& Source,
	const TDesC& Reason, TInt Code)
{
	iContext.ReportActiveError(Source, Reason, Code);
}

EXPORT_C MApp_context* GetContext()
{
	return MApp_context::Static();
}

EXPORT_C MApp_context* MContextBase::GetContext() const
{
	return &iContext;
}

EXPORT_C CBBSession* MContextBase::BBSession()
{
	return iContext.BBSession();
}

_LIT(KSep, "::");
_LIT8(KSep8, "::");

TInt CApp_contextImpl::PushCallStack(const TDesC8& Name)
{
	return PushCallStack(KNullDesC8, Name);
}

_LIT8(KSpace8, " ");

TPtrC8 CApp_contextImpl::GetCurrentClass()
{
	if (iCallStackCurrent==iCallStackZero) {
		return TPtrC8(0, 0);
	}
	TInt* pd=(TInt*)(iCallStackCurrent - 4);
	char* p= iCallStackZero + *pd;
	TInt len=(iCallStackCurrent - p - 4);
	TPtr8 ret( (TUint8*)(p), len, len + 
		(iCallStackZero+iCallStackSize-iCallStackCurrent-8) );
	TInt seppos=ret.Find(KSep8);
	if (seppos!=KErrNotFound) {
		if (seppos > ret.Length() ) {
			ret.SetLength(0);
		} else {
			ret.SetLength( seppos );
		}
	}
	return ret;
}

TInt CApp_contextImpl::PushCallStack(const TDesC8& Class, const TDesC8& Func)
{
	if (iDebugCallStack) {
		if (iDebugLogIsOpen) {
			TBuf<50> tmp;
			TBuf<128> msg=_L("-> ");
			tmp.Copy(Class.Left(50));
			msg.Append(tmp);
			msg.Append(_L("::"));
			tmp.Copy(Func.Left(50));
			msg.Append(tmp);
			DebugLog(msg);
		}
	}
	if (!iCallStackZero) return KErrNotFound;
	TInt Length=Class.Length()+Func.Length()+2;
	if (iCallStackCurrent+Length+8 > iCallStackZero+iCallStackSize) return KErrNoMemory;

	TInt len, added;
	len=Length;
	// align on 4 byte boundaries
	added=(4-(len % 4));
	if (added==4) added=0;
	len+=added;

	TPtr8 pt( (TText8*)iCallStackCurrent, len);
	pt=Class;
	pt.Append(KSep8);
	pt.Append(Func);

	while(added>0) {
		pt.Append(KSpace8);
		added--;
	}

	TInt* pp=(TInt*) (iCallStackCurrent+len);
#ifdef DEBUG_CALLSTACK
	//FIXME 8->16
	TBuf<256> msg;
	msg.Format(_L("PushCallStack name at %d length %d link at %d next at %d: %S::%S"),
		(TInt)iCallStackCurrent, TInt(len), 
		(TInt)pp, (TInt)(iCallStackCurrent+len + 4), &Class, &Func);
	RDebug::Print(msg);
#endif

	*pp=iCallStackCurrent-iCallStackZero;
	iCallStackCurrent+=len + 4;

	*((TInt*) (iCallStackZero-4)) = (iCallStackCurrent - iCallStackZero);

	return KErrNone;
}

TPtr8 CApp_contextImpl::PopCallStack()
{
	if (iCallStackCurrent==iCallStackZero) {
		return TPtr8(0, 0);
	}
	TInt* pd=(TInt*)(iCallStackCurrent - 4);
	char* p= iCallStackZero + *pd;
	TInt len=(iCallStackCurrent - p - 4);
	TPtr8 ret( (TUint8*)(p), len, len + 
		(iCallStackZero+iCallStackSize-iCallStackCurrent-8) );

	if (iDebugCallStack) {
		if (iDebugLogIsOpen) {
			TBuf<128> msg=_L("<- ");
			TBuf<100> tmp; tmp.Copy(ret.Left(100));
			msg.Append(tmp);
			DebugLog(msg);
		}
	}
#ifdef DEBUG_CALLSTACK
	TBuf<200> msg;
	msg.Format(_L("PopCallStack name at %d length %d link at %d next at %d: %S"),
		(TInt)p, TInt(len*2), 
		(TInt)pd, (TInt)(iCallStackCurrent), &ret);
	RDebug::Print(msg);
#endif

	iCallStackCurrent=p;
	*((TInt*) (iCallStackZero-4)) = (iCallStackCurrent - iCallStackZero);
	return ret;
}

void CApp_contextImpl::ResetCallStack()
{
	iCallStackCurrent=iCallStackZero;
	*((TInt*) (iCallStackZero-4)) = 0;
}

class TFormattedStack : public MCallStackVisitor {
public:
	TFormattedStack(HBufC* Into, const TDesC& Prefix) : iInto(Into), iPrefix(Prefix) { }
private:
	virtual void BeginStack()
	{
		iInto->Des().Zero();
		iInto->Des().Append(iPrefix);
		iInto->Des().Append(_L(":"));
	}
	virtual void VisitItem(const TDesC& Name)
	{
		iInto->Des().Append(Name);
		iInto->Des().Append(_L("|"));
	}
	virtual void EndStack()
	{
	}

	HBufC* iInto;
	const TDesC& iPrefix;
};

HBufC* CApp_contextImpl::GetFormattedCallStack(const TDesC& prefix)
{
	const char* aStack=iCallStackZero-4;
	const char* current=aStack+4+*(TInt*)aStack;
	aStack+=4;

	if (current==aStack) return 0;
	HBufC* ret=0;
	TInt len=((current-aStack)) + prefix.Length() + 1;
	CC_TRAPD(err, ret=HBufC::NewL( len ));
	if (err!=KErrNone) return 0;

	TFormattedStack f(ret, prefix);
	IterateStack(f);

#ifdef __WINS__
	if (ret->Des().Length()>128) {
		RDebug::Print(ret->Des().Left(128));
	} else {
		RDebug::Print(ret->Des());
	}
#endif
	return ret;
}


void CApp_contextImpl::IterateStack(MCallStackVisitor& aVisitor, const char* aStack)
{
	if (aStack==0) aStack=iCallStackZero-4;

	TBuf<202> item;
	aVisitor.BeginStack();
	const char* current=aStack+4+*(TInt*)aStack;
	aStack+=4;
	while (current!=aStack) {
		const char *prev=current -4;
		TInt* p=(TInt*)(prev);
		current=aStack + *p;
		TInt len=(prev-current);
		TPtrC8 n( (TUint8*)current, len);
		item.Copy(n);
		aVisitor.VisitItem(item);
	}
	aVisitor.EndStack();
}
		
MErrorInfo* CApp_contextImpl::GetLastErrorInfo(TErrorCode aCode)
{
	if (aCode.iCode==0) {
		delete iLastErrorInfo; iLastErrorInfo=0;
		return 0;
	}
	if (! iLastErrorInfo ) {
		CC_TRAPD(err, iLastErrorInfo=CErrorInfo::NewL(aCode, 0, 0, 0,
			EBug, EError, 0));
	}
	return iLastErrorInfo;
}

MErrorInfo* CApp_contextImpl::GetLastErrorInfo()
{
	return iLastErrorInfo;
}

MReporting& CApp_contextImpl::Reporting()
{
	return *this;
}

MErrorInfoManager& CApp_contextImpl::ErrorInfoMgr()
{
	return *this;
}
MCallStack& CApp_contextImpl::CallStackMgr()
{
	return *this;
}

void CApp_contextImpl::ResetLastErrorInfo()
{
	delete iLastErrorInfo;
	iLastErrorInfo=0;
}

EXPORT_C TCallStackItem::TCallStackItem(const TDesC8& Name, MApp_context* Context)
{
	if (!Context) {
		iCallStack=0;
		return;
	}
	iCallStack=&(Context->CallStackMgr());

	TInt err=iCallStack->PushCallStack(Name.Left(100));
	if (err!=KErrNone) {
		iCallStack=0;
	} else {
#ifdef FUNCTION_LEVEL_STATISTICS		
		iStartTicks=User::TickCount();
#endif
	}
}

EXPORT_C TCallStackItem::TCallStackItem(const TDesC8& Class, const TDesC8& Func, MApp_context* Context)
{
	if (!Context) {
		iCallStack=0;
		return;
	}
	iCallStack=&(Context->CallStackMgr());

	TInt err=iCallStack->PushCallStack(Class.Left(100), Func.Left(100));
	if (err!=KErrNone) {
		iCallStack=0;
	} else {
#ifdef FUNCTION_LEVEL_STATISTICS		
		iStartTicks=User::TickCount();
#endif
	}
}

EXPORT_C TCallStackItem::~TCallStackItem()
{
	if (!iCallStack) return;
#ifdef FUNCTION_LEVEL_STATISTICS
	TPtr8 m=iCallStack->PopCallStack();
	if (m.Length()+26 < m.MaxLength()) {
		TUint elapsed=User::TickCount()-iStartTicks;
		m.Append(_L(": "));
		m.AppendNum(elapsed);
		m.Append(_L(" ELAPSED"));
		iAppContext->DebugLog(m);
	}
#else
	iCallStack->PopCallStack();
#endif

}

EXPORT_C void MoveIntoDataDirL(MApp_context& Context, const TDesC& FileName)
{
	TFileName appfile, datafile;

	appfile.Format(_L("%S%S"), &Context.AppDir(), &FileName);
	datafile.Format(_L("%S%S"), &Context.DataDir(), &FileName);

	// if appdir and datadir are the same
	if (! appfile.CompareF(datafile)) return;

	if (BaflUtils::FileExists(Context.Fs(), appfile)) {
		User::LeaveIfError(BaflUtils::CopyFile(Context.Fs(), appfile, datafile));
		User::LeaveIfError(BaflUtils::DeleteFile(Context.Fs(), appfile));
	}
}

#ifdef __WINS__
EXPORT_C void StartStarterL(const TDesC& StackName, TUid AppUid, bool /*CheckForRunning*/, RWsSession& Ws)
#else
EXPORT_C void StartStarterL(const TDesC& StackName, TUid AppUid, bool CheckForRunning)
#endif
{
	TBuf<100> cmdline=StackName;;
	cmdline.Append(_L("*")); cmdline.AppendNum((TInt)AppUid.iUid);
#ifdef __WINS__
	const TUid starter_uid= KUidstarter;
	RApaLsSession ls;
	User::LeaveIfError(ls.Connect());
	CleanupClosePushL(ls);

	TApaTaskList tl(Ws);
	TApaTask t=tl.FindApp(KUidstarter);
	if (! t.Exists()) {
		TThreadId dummy;
		User::LeaveIfError( ls.StartDocument(cmdline, starter_uid, dummy) );
	}
	CleanupStack::PopAndDestroy();
#else
	TFileName fnAppPath = _L("c:\\system\\apps\\starter\\starter.exe");

	{
		TFindProcess f(_L("*"));
		TFullName t;
		RProcess r;
		while (f.Next(t)==KErrNone) {
			r.Open(t);
			CleanupClosePushL(r);
			//LogAppEvent(t); LogAppEvent(r.FileName());
			if (r.FileName().Length()>0 && (! r.FileName().Mid(1).CompareF( fnAppPath.Mid(1) )) ) {
				// already running
				bool running=false;
				if (CheckForRunning && r.CommandLineLength()==0) running=true;
				else {
					HBufC* c=HBufC::NewLC(r.CommandLineLength());
					TPtr16 d=c->Des();
					r.CommandLine(d);
					if (! (d.Compare(cmdline)) ) {
						running=true;
					}
					CleanupStack::PopAndDestroy();
				}
				if (running) {
					CleanupStack::PopAndDestroy();
					return;
				}
			}
			CleanupStack::PopAndDestroy();
		}
	}

	{
		RProcess server;

		if (server.Create(fnAppPath, cmdline) != KErrNone) {
			fnAppPath.Replace(0, 1, _L("e"));
			User::LeaveIfError(server.Create(fnAppPath, cmdline));
		}
		CleanupClosePushL(server);
		server.Resume();

		CleanupStack::PopAndDestroy(); // server
	}
#endif
}

EXPORT_C TBool HasMMC(RFs& aFs)
{
	TDriveInfo i;
	if (aFs.Drive(i, EDriveE)==KErrNone) {
		if (i.iType!=EMediaNotPresent &&
			i.iType!=EMediaUnknown &&
			i.iType!=EMediaCdRom &&
			i.iType!=EMediaRom) {
			// memory card
			return ETrue;
		}
	}
	return EFalse;
}

EXPORT_C MReporting& MContextBase::Reporting()
{
	return iContext.Reporting();
}

EXPORT_C MBBDataFactory* MContextBase::BBDataFactory()
{
	return iContext.BBDataFactory();
}

EXPORT_C MErrorInfoManager& MContextBase::ErrorInfoMgr()
{
	return iContext.ErrorInfoMgr();
}

EXPORT_C MCallStack& MContextBase::CallStackMgr()
{
	return iContext.CallStackMgr();
}
