/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

/*
 * Concepts:
 * !Making calls!
 */

#include "break.h"
#include "phonehelper.h"
#include "app_context.h"
#include <txtrich.h>
#include <SenduiMtmUids.h>
#include <MsgBioUids.h>
#include <msvapi.h>
#include <msvids.h>

class dummyhandler: public MMsvSessionObserver {
public:
	virtual void HandleSessionEventL(TMsvSessionEvent, TAny*, TAny*, TAny*) { }
};

EXPORT_C void phonehelper::ConstructL()
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("ConstructL"));

	iSendAppUi = CSendAppUi::NewL(0);

#if !defined(__WINS__) || defined(__S60V2__)
	iStatus=KErrNone;
	
	User::LeaveIfError( server.Connect() );
	iServerOpen = ETrue;

	// load a phone profile
#  ifndef __WINS__
		_LIT(KGsmModuleName, "phonetsy.tsy");
#  else
		_LIT(KGsmModuleName, "mm.tsy");
#  endif
	User::LeaveIfError( server.LoadPhoneModule(KGsmModuleName) );

	TInt numberPhones;
	User::LeaveIfError(server.EnumeratePhones(numberPhones));
	if (numberPhones < 1) {
		User::Leave(KErrNotFound);
	}

	//Get info about the first available phone
	RTelServer::TPhoneInfo info;
	User::LeaveIfError(server.GetPhoneInfo(0, info));
	User::LeaveIfError( phone.Open( server, info.iName ) );
	iPhoneOpen = ETrue;

	// RPhone::TLineInfo lineinfo;
	// phone.GetLineInfo(0, lineinfo);
	// line.Open(phone, lineinfo.iName);

#endif
	CActiveScheduler::Add(this); 
}

EXPORT_C void phonehelper::CheckedRunL()
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("CheckedRunL"));

	if (iCallOpen) {
		iCallOpen=EFalse;
		call.Close();
	}
	if (iLineOpen) {
		iLineOpen=EFalse;
		line.Close();
	}
	
}

EXPORT_C void phonehelper::DoCancel()
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("DoCancel"));

	if (iCallOpen) {
		call.DialCancel();
		iCallOpen = EFalse;
		call.Close();
	}

	if (iLineOpen) {
		iLineOpen = EFalse;
		line.Close();
	}
}

EXPORT_C phonehelper::~phonehelper()
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("~phonehelper"));
	Cancel();

	if (iPhoneOpen) {
		iPhoneOpen=EFalse;
		phone.Close();
	}

	if (iServerOpen) {
#ifndef __WINS__
		_LIT(KGsmModuleName, "phonetsy.tsy");
#else
#  ifdef __S60V2__
		_LIT(KGsmModuleName, "mm.tsy");
#  else
		_LIT(KGsmModuleName, "hayes.tsy");
#  endif
#endif
		server.UnloadPhoneModule( KGsmModuleName );
		iServerOpen=EFalse;
		server.Close();
	}
	delete iSendAppUi;
}

/*RCall::TStatus phonehelper::line_status()
{
	CALLSTACKITEM_N(_CL("RCall"), _CL("TStatus"));


	RLine::TLineInfo lineinfo;
	#ifndef __WINS__
		line.GetInfo(lineinfo);
	#else
		lineinfo.iStatus=RCall::EStatusIdle;
	#endif
	return lineinfo.iStatus;
}*/

EXPORT_C TInt phonehelper::make_callL(const TDesC& number)
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("make_callL"));

	if (aLog) {
		aLog->write_time();
		aLog->write_to_output(_L("make call to "));
		aLog->write_to_output(number);
		aLog->write_nl();
	}

	// The status isn't reliable if no calls
	// have been made yet after startup
	/*
	if (line_status()!=RCall::EStatusIdle) {
		return -1;
	}

	*/


	if (IsActive()) return -2;

	//Get info about the first available line
	TInt numberLines;
	User::LeaveIfError(phone.EnumerateLines(numberLines));
	if (numberLines <1) {
		User::Leave(KErrNotFound);
	}

	RPhone::TLineInfo lineInfo;
	TInt i;
	for (i=0; i<numberLines; i++) {
		User::LeaveIfError(phone.GetLineInfo(i, lineInfo));
#ifdef __WINS__
		if ( lineInfo.iLineCapsFlags & RPhone::KCapsVoice ) 
			break;
#else
		break;
#endif
	}
	//if ( ! (lineInfo.iLineCapsFlags & RPhone::KCapsVoice) ) 
		//User::Leave(KErrNotSupported);
	User::LeaveIfError(line.Open(phone, lineInfo.iName));
	iLineOpen = ETrue;

	TInt ret=call.OpenNewCall(line);
	iCallOpen = ETrue;
	if (ret==KErrNone) {
		call.Dial(iStatus, number);
		SetActive();
	}
	return ret;
}

EXPORT_C void phonehelper::send_sms(CDesCArrayFlat* recip, CDesCArrayFlat* alias)
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("send_sms"));

	if (aLog) {
		aLog->write_time();
		aLog->write_to_output(_L("write sms to "));
		for (int i=0; i<recip->Count(); i++)
		{
			aLog->write_to_output( (*recip)[i] );
			aLog->write_to_output(_L(" "));
		}
		aLog->write_nl();
	}

	CParaFormatLayer* paraf=CParaFormatLayer::NewL();
	CleanupStack::PushL(paraf);

	CCharFormatLayer* charf=CCharFormatLayer::NewL();
	CleanupStack::PushL(charf);

	CRichText* body;
	body=CRichText::NewL(paraf, charf);
	CleanupStack::PushL(body);
		
	iSendAppUi->CreateAndSendMessageL(KSenduiMtmSmsUid, body, 0, KNullUid, recip, alias);
	CleanupStack::PopAndDestroy(3); // body, paraf, charf
}

EXPORT_C void phonehelper::send_mms(CDesCArrayFlat* recip, CDesCArrayFlat* alias)
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("send_mms"));

	if (aLog) {
		aLog->write_time();
		aLog->write_to_output(_L("write mms to "));
		for (int i=0; i<recip->Count(); i++)
		{
			aLog->write_to_output( (*recip)[i] );
			aLog->write_to_output(_L(" "));
		}
		aLog->write_nl();
	}

	CParaFormatLayer* paraf=CParaFormatLayer::NewL();
	CleanupStack::PushL(paraf);

	CCharFormatLayer* charf=CCharFormatLayer::NewL();
	CleanupStack::PushL(charf);

	CRichText* body;
	body=CRichText::NewL(paraf, charf);
	CleanupStack::PushL(body);
		
	iSendAppUi->CreateAndSendMessageL(KSenduiMtmMmsUid, body, 0, KNullUid, recip, alias);
	CleanupStack::PopAndDestroy(3); // body, paraf, charf
}

EXPORT_C void phonehelper::send_email(CDesCArrayFlat* recip, CDesCArrayFlat* alias)
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("send_email"));

	if (aLog) {
		aLog->write_time();
		aLog->write_to_output(_L("write email to "));
		for (int i=0; i<recip->Count(); i++)
		{
			aLog->write_to_output( (*recip)[i] );
			aLog->write_to_output(_L(" "));
		}
		aLog->write_nl();
	}

	CParaFormatLayer* paraf=CParaFormatLayer::NewL();
	CleanupStack::PushL(paraf);

	CCharFormatLayer* charf=CCharFormatLayer::NewL();
	CleanupStack::PushL(charf);

	CRichText* body;
	body=CRichText::NewL(paraf, charf);
	CleanupStack::PushL(body);
		
	iSendAppUi->CreateAndSendMessageL(KSenduiMtmSmtpUid, body, 0, KNullUid, recip, alias);
	CleanupStack::PopAndDestroy(3); // body, paraf, charf
}

EXPORT_C TBool phonehelper::mailbox_defined()
{
	CALLSTACKITEM_N(_CL("phonehelper"), _CL("mailbox_defined"));

	TBool mailbox_defined = EFalse;

	dummyhandler * dummy=new (ELeave) dummyhandler;
	CleanupStack::PushL(dummy);
	CMsvSession * session = CMsvSession::OpenSyncL(*dummy);
	CleanupStack::PushL(session);
	CMsvEntry* entry = CMsvEntry::NewL(*session, KMsvRootIndexEntryId ,TMsvSelectionOrdering());
	CleanupStack::PushL(entry);
	CMsvEntrySelection* imap = entry->ChildrenWithMtmL(KSenduiMtmImap4Uid);
	CleanupStack::PushL(imap);
	CMsvEntrySelection* pop = entry->ChildrenWithMtmL(KSenduiMtmPop3Uid);
	CleanupStack::PushL(pop);
	if ((imap->Count() >0) || (pop->Count()>0)) mailbox_defined = ETrue;
	CleanupStack::PopAndDestroy(5); //dummy, session, entry
	return mailbox_defined;
}


