#include "contextdoc.h"
#include "app_context_impl.h"
#include "bb_settings.h"
#include <aknapp.h>
#include "cc_schedulers.h"
#include "cbbsession.h"
#include "reporting.h"
#include "callstack.h"
#include "symbian_auto_ptr.h"
#include "break.h"
#include <eikenv.h>
#include "context_uids.h"

EXPORT_C TInt MContextDocument::InitializationSteps()
{
	return 5;
}

EXPORT_C void MContextDocument::StepDone()
{
}

EXPORT_C void MContextDocument::ConstructL(class CApaApplication* aApp, 
	const class MDefaultSettings& aDefaultSettings,
	const struct TTupleName& aSettingTupleName,
	const TDesC& aDebugLog,
	const TDesC& aChunkName)
{
	iContext=CApp_context::NewL(false, aChunkName);
	StepDone();
#ifndef __WINS__
	{ CC_TRAPD(errs, StartStarterL(aChunkName, aApp->AppDllUid(), true)); }
#else
	if (0 && aApp->AppDllUid() == KUidcontext_log) {
		// can't pass arguments on the emulator, so only works for context_log
		{ CC_TRAPD(errs, StartStarterL(aChunkName, aApp->AppDllUid(), true, CEikonEnv::Static()->WsSession())); }
	}
#endif
	StepDone();

	iContext->Reporting().SetState(_L("Set data dir"));
	iContext->SetDataDir(_L("c:\\system\\data\\context\\"), false);
	iContext->Reporting().SetState(_L("Set debug log"));
	iContext->SetDebugLog(_L("context"), aDebugLog);

	TRAPD(err, InnerConstructL(aApp, aDefaultSettings, aSettingTupleName));
	if (err!=KErrNone) {
		iContext->Reporting().UserErrorLog(_L("Failed to construct application"), err);
		auto_ptr<HBufC> s(iContext->CallStackMgr().GetFormattedCallStack(KNullDesC));
		iContext->Reporting().UserErrorLog(*s);
		User::Leave(err);
	}
}

void MContextDocument::InnerConstructL(class CApaApplication* aApp, 
	const class MDefaultSettings& aDefaultSettings,
	const struct TTupleName& aSettingTupleName)
{

	iContext->Reporting().SetState(_L("Create scheduler"));
	iDebugScheduler=new (ELeave) CDebugAppScheduler(*iContext, CCoeEnv::Static());
	iBuiltinScheduler=CActiveScheduler::Replace(iDebugScheduler);
	StepDone();

#ifndef __WINS__
	{
		iContext->Reporting().SetState(_L("set app dir"));
		TParsePtrC parse(aApp->DllName());
		iContext->SetAppDir(parse.DriveAndPath());
	}
#endif
	iContext->Reporting().SetState(_L("Create BB settings"));

	auto_ptr<CApp_context> ctx(CApp_context::NewL(true, _L("BlackBoardServer")));
	TInt err=0;
	CBlackBoardSettings* s=0;
	CC_TRAP(err, s=CBlackBoardSettings::NewL(*iContext, aDefaultSettings, aSettingTupleName));
	if (err==KErrServerTerminated) {
		auto_ptr<HBufC> stack(0);
		
		CC_TRAPD(err, stack.reset(ctx->CallStackMgr().GetFormattedCallStack(_L("BlackboardCrash"))));
		if (err==KErrNone && stack.get()!=0) {
			iContext->Reporting().UserErrorLog(*stack);
		}
		User::Leave(KErrServerTerminated);
	} else if (err!=KErrNone) {
		User::Leave(err);
	}

	iContext->SetSettings(s);
	StepDone();

	iContext->Reporting().SetState(_L("Create BB factory"));
	iBBDataFactory=CBBDataFactory::NewL();
	iBBSession=CBBSession::NewL(*iContext, iBBDataFactory);
	iContext->SetBBSession(iBBSession);
	iContext->SetBBDataFactory(iBBDataFactory);
	StepDone();
}


EXPORT_C MContextDocument::~MContextDocument()
{
#ifdef __WINS__
	User::Check();
#endif
	if (iBuiltinScheduler) {
#ifdef __WINS__
		iDebugScheduler->CheckThatEmpty();
#endif
		CActiveScheduler::Replace(iBuiltinScheduler);
	}
	delete iDebugScheduler;

	delete iBBSession;
	delete iBBDataFactory;
	delete iContext;

#ifdef __WINS__
	User::Check();
#endif
}
