#include "break.h"
#ifndef __S80__
#include <aknutils.h>
#include <aknviewappui.h>
#endif
#include <txtrich.h>
#include <errorinfo.h>
#include "cu_errorui.h"
#include <eikrted.h>
#include <app_context.h>
#include <symbian_auto_ptr.h>
#include <bautils.h>
#include <contextui.rsg>
#include <sendui.h>
#include "contextui.hrh"
#include "raii_f32file.h"
#include  <SENDNORM.RSG>
#include <eikenv.h>

void AppendFormatted(CRichText& aInto, const TDesC& aText, const TRgb& aColor)
{
	TInt pos=aInto.DocumentLength();
#ifndef __S80__
	TFontSpec fontspec = LatinPlain12()->FontSpecInTwips();
#else
	TFontSpec fontspec = CEikonEnv::Static()->DenseFont()->FontSpecInTwips();
#endif
	TCharFormat charFormat( fontspec.iTypeface.iName, fontspec.iHeight );
	TCharFormatMask charFormatMask;
	charFormat.iFontPresentation.iTextColor=aColor;
	charFormatMask.SetAttrib(EAttColor);

	aInto.InsertL(pos, aText);
	aInto.ApplyCharFormatL(charFormat, charFormatMask, pos, aText.Length());

	CParaFormat paraFormat;
	TParaFormatMask paraFormatMask;
	aInto.GetParaFormatL(&paraFormat, paraFormatMask, pos, aText.Length());
	paraFormatMask.SetAttrib(EAttLineSpacingControl);
	paraFormatMask.SetAttrib(EAttLineSpacing);
	// paraFormatMask.SetAttrib(EAttAlignment);
	paraFormat.iLineSpacingControl = CParaFormat::ELineSpacingExactlyInTwips;
	paraFormat.iLineSpacingInTwips = 115;
	// paraFormat.iHorizontalAlignment = aAlignment;
	aInto.ApplyParaFormatL(&paraFormat, paraFormatMask, pos, aText.Length());

}

void UserFriendlyIntoRichText(CRichText& aInto, const MErrorInfo& aFrom)
{
	TBuf<14> pre;
	TRgb color;
	switch (aFrom.Severity()) {
	case EInfo:
		pre=_L("[Info] ");
		color=KRgbBlue;
		break;
	case EWarning:
		pre=_L("[Warning] ");
		color=KRgbYellow;
		break;
	case EError:
		pre=_L("[Error] ");
		color=KRgbRed;
		break;
	case ECorrupt:
		pre=_L("[Critical] ");
		color=KRgbDarkRed;
		break;
	}

	AppendFormatted(aInto, pre, color);
	AppendFormatted(aInto, aFrom.UserMessage(), color);
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);

	switch (aFrom.ErrorType()) {
	case EBug:
		AppendFormatted(aInto, _L("Contact support."), color);
		break;
	case EInputData:
		AppendFormatted(aInto, _L("Please correct the input."), color);
		break;
	case ETemporary:
		AppendFormatted(aInto, _L("Please try again."), color);
		break;
	case ELocalEnvironment:
		AppendFormatted(aInto, _L("There is a problem with the setup on the phone."), color);
		break;
	case ERemote:
		AppendFormatted(aInto, _L("There is a problem with the remote system."), color);
		break;
	}
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);

}

void TechnicalIntoRichText(CRichText& aInto, const MErrorInfo& aFrom)
{
	AppendFormatted(aInto, _L("Technical information:"), KRgbBlack);
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);
	AppendFormatted(aInto, aFrom.TechMessage(), KRgbBlack);
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);

	AppendFormatted(aInto, _L("Error code: "), KRgbBlack);
	TBuf<24> code; code.AppendNum( aFrom.ErrorCode().iUid, EHex );
	code.Append(_L(" "));
	code.AppendNum( aFrom.ErrorCode().iCode );
	AppendFormatted(aInto, code, KRgbBlack);
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);

	AppendFormatted(aInto, _L("Stack trace: "), KRgbBlack);
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);
	AppendFormatted(aInto, aFrom.StackTrace(), KRgbBlack);
	aInto.InsertL(aInto.DocumentLength(), CEditableText::EParagraphDelimiter);
}

EXPORT_C void ErrorIntoRichTextL(CRichText& aInto, const MErrorInfo& aFrom)
{
	TInt	nodecount=0;

	const MErrorInfo* list_i=&aFrom;
	while (list_i) {
		const MErrorInfo* nest_i=list_i;
		while (nest_i) {
			nodecount++;
			UserFriendlyIntoRichText(aInto, *nest_i);
			nest_i=nest_i->InnerError();
		}
		list_i=list_i->NextError();
	}
	if (nodecount==1) {
		TechnicalIntoRichText(aInto, aFrom);
	} else {
		list_i=&aFrom;
		while (list_i) {
			const MErrorInfo* nest_i=list_i;
			while (nest_i) {
				nodecount++;
				UserFriendlyIntoRichText(aInto, *nest_i);
				TechnicalIntoRichText(aInto, *nest_i);
				nest_i=nest_i->InnerError();
			}
			list_i=list_i->NextError();
		}
	}
}

class CErrorInfoContainer: public CCoeControl, public MContextBase {
public:
	void ConstructL(const TRect& aRect) {
		CreateWindowL();

		iText=new (ELeave) CEikRichTextEditor();
#ifndef __S80__
		iText->SetMopParent(this);
#endif
		const TInt edwinFlags = EEikEdwinInclusiveSizeFixed|
			EEikEdwinNoAutoSelection|
			EEikEdwinDisplayOnly|
			EEikEdwinReadOnly|
			EEikEdwinLineCursor|
			EEikEdwinNoHorizScrolling
#ifndef __S80__
		| EEikEdwinAvkonDisableCursor;
#else
			;
#endif
		iText->ConstructL( this, 10, 2000, edwinFlags );
#ifndef __S80__
		iText->SetAknEditorFlags(edwinFlags);
#endif
		iText->SetContainerWindowL(*this);

		TRect r=aRect; r.iTl.iX=0; r.iTl.iY=0;
		iText->SetRect( r );
		iText->CreatePreAllocatedScrollBarFrameL();
		iText->SetReadOnly(ETrue);
		iText->SetFocusing(ETrue);
		iText->SetFocus(ETrue);
		iText->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, 
			CEikScrollBarFrame::EAuto);

		SetRect(aRect);
		ActivateL();
	}
	TInt CountComponentControls() const {
		return 1;
	}
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType )
	{
		switch(aKeyEvent.iCode) {
		case EKeyUpArrow:{
			TInt pos=iText->CursorPos();
			iText->MoveCursorL(TCursorPosition::EFPageUp, EFalse);
			iText->MoveCursorL(TCursorPosition::EFLineBeg, EFalse);
			if (pos!=iText->CursorPos()) {
				iText->MoveDisplayL(TCursorPosition::EFPageUp);
				iText->DrawNow();
			}
			iText->UpdateScrollBarsL();
			iText->ScrollBarFrame()->DrawScrollBarsNow();
			return EKeyWasConsumed;
		}
			
		case EKeyDownArrow:{
			TInt pos=iText->CursorPos();
			iText->MoveCursorL(TCursorPosition::EFPageDown, EFalse);
			iText->MoveCursorL(TCursorPosition::EFLineBeg, EFalse);
			if (pos!=iText->CursorPos()) {
				iText->MoveDisplayL(TCursorPosition::EFPageDown);
				iText->DrawNow();
			}
			iText->UpdateScrollBarsL();
			iText->ScrollBarFrame()->DrawScrollBarsNow();
			return EKeyWasConsumed;
		}
			
		default:
			break;
		}
		
		return EKeyWasNotConsumed;
	}   
	CCoeControl* ComponentControl(TInt aIndex) const
	{
		return iText;
	}

	void HandleTextChangedL() {
		iText->HandleTextChangedL();
		iText->UpdateScrollBarsL();
		iText->ScrollBarFrame()->DrawScrollBarsNow();
	}
	CRichText* RichText() {
		return iText->RichText();
	}
	TInt TextLength() {
		return iText->TextLength();
	}
	~CErrorInfoContainer() {
		delete iText;
	}
private:
	CEikRichTextEditor* iText;
};

class CErrorInfoViewImpl : public CErrorInfoView, public MContextBase {
private:
	CErrorInfoViewImpl();
	void ConstructL();

	virtual void ShowWithData(const MErrorInfo& aInfo);
	
	TUid Id() const;

        void HandleCommandL(TInt aCommand);
        void DoActivateL(const TVwsViewId& aPrevViewId,
		TUid aCustomMessageId,
		const TDesC8& aCustomMessage);
        void DoDeactivate();
	void DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane);
	void OpenInSendL(TInt aCommand);

	friend class CErrorInfoView;


	MErrorInfo*		iErrorInfo;
	TVwsViewId	iPrevView;
	TInt		iResource;
	CErrorInfoContainer* iContainer;
	CSendAppUi*	iSendUi;
public:
	virtual ~CErrorInfoViewImpl();
};

EXPORT_C CErrorInfoView* CErrorInfoView::NewL()
{
	CALLSTACKITEM_N(_CL("CErrorInfoView"), _CL("NewL"));

	auto_ptr<CErrorInfoViewImpl> ret(new (ELeave) CErrorInfoViewImpl);
	ret->ConstructL();
	return ret.release();
}
CErrorInfoViewImpl::CErrorInfoViewImpl() { }

void CErrorInfoViewImpl::ConstructL()
{
	CALLSTACKITEM_N(_CL("CErrorInfoViewImpl"), _CL("ConstructL"));
#ifndef __WINS__
	TFileName resfile=_L("c:\\System\\data\\contextui.rsc");
	if (! BaflUtils::FileExists(iEikonEnv->FsSession(), resfile) ) {
		resfile.Replace(0, 1, _L("e"));
	}
#else
	TFileName resfile=_L("z:\\System\\data\\contextui.rsc");
	BaflUtils::NearestLanguageFile(iEikonEnv->FsSession(), resfile); //for localization
#endif
	CC_TRAPD(err, iResource=iEikonEnv->AddResourceFileL(resfile));
	User::LeaveIfError(err);
	BaseConstructL( R_ERRORINFO_VIEW );

	iSendUi = CSendAppUi::NewL(EContextUICmdSendUi, 0);
}

TUid CErrorInfoViewImpl::Id() const
{
	CALLSTACKITEM_N(_CL("CErrorInfoViewImpl"), _CL("Id"));

	return KErrorInfoView;
}

void CErrorInfoViewImpl::HandleCommandL(TInt aCommand)
{
	CALLSTACKITEM_N(_CL("CErrorInfoViewImpl"), _CL("HandleCommandL"));
	if (aCommand==EAknSoftkeyBack || aCommand==EAknSoftkeyClose) {
		ActivateViewL(iPrevView);
	} else {
		OpenInSendL(aCommand);
	}
}

void CErrorInfoViewImpl::OpenInSendL(TInt aCommandId)
{
	TUid mtm_uid=iSendUi->MtmForCommand(aCommandId) ;
	TSendingCapabilities cap=iSendUi->MtmCapabilitiesL(mtm_uid);
	CRichText* body=0;

	auto_ptr<CDesCArrayFlat> attach(new (ELeave) CDesCArrayFlat(1));
	TFileName body_filen;

	if (cap.iFlags & TSendingCapabilities::ESupportsBodyText) {
		body=iContainer->RichText();
	} else {
		body_filen=DataDir();
		body_filen.Append( _L("errorinfo") );
		body_filen.Append(_L(".txt"));
		Fs().Delete(body_filen);
		RAFile f;
		f.ReplaceLA(Fs(), body_filen, EFileWrite|EFileShareAny);
		const TDesC& errortext = iContainer->RichText()->Read(0, iContainer->TextLength());

		TPtrC8 body( (TText8*)errortext.Ptr(), errortext.Length()*2);
		User::LeaveIfError(f.Write(body));
		attach->AppendL(body_filen);
	}

#ifndef __S80__
	iSendUi->CreateAndSendMessageL(aCommandId, body, attach.get(),
		KNullUid, 0, 0, EFalse);
#else
	iSendUi->CreateAndSendMessageL(aCommandId, body, attach.get(),
		KNullUid, 0, 0);
#endif

}

void CErrorInfoViewImpl::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane)
{
	if (aResourceId == R_SENDUI_MENU) {
		iSendUi->DisplaySendCascadeMenuL(*aMenuPane);
	} else if (aResourceId == R_ERRORUI_MENU) {
		TInt flags=TSendingCapabilities::ESupportsAttachmentsOrBodyText;
		TSendingCapabilities c( 0, 0, flags);
		iSendUi->DisplaySendMenuItemL(*aMenuPane, 0, c);
	}
}

void CErrorInfoViewImpl::DoActivateL(const TVwsViewId& aPrevViewId,
	TUid /*aCustomMessageId*/,
	const TDesC8& /*aCustomMessage*/)
{
	CALLSTACKITEM_N(_CL("CErrorInfoViewImpl"), _CL("DoActivateL"));

	iPrevView=aPrevViewId;
	if (!iContainer) {
		iContainer=new (ELeave) CErrorInfoContainer();
#ifndef __S80__
		iContainer->SetMopParent(this);
#endif
		iContainer->ConstructL(ClientRect());

		if (iErrorInfo) {
			ErrorIntoRichTextL( *iContainer->RichText(), *iErrorInfo );
			iContainer->HandleTextChangedL();
			delete iErrorInfo; iErrorInfo = 0;
		}
		AppUi()->AddToStackL( *this, iContainer );
        } 
}

void CErrorInfoViewImpl::DoDeactivate()
{
	CALLSTACKITEM_N(_CL("CErrorInfoViewImpl"), _CL("DoDeactivate"));

	if ( iContainer )
        {
		AppUi()->RemoveFromViewStack( *this, iContainer );
        }
	
	delete iContainer; iContainer = 0;
	delete iErrorInfo; iErrorInfo = 0;
}

CErrorInfoViewImpl::~CErrorInfoViewImpl()
{
	CALLSTACKITEM_N(_CL("CErrorInfoViewImpl"), _CL("~CErrorInfoViewImpl"));

	if (iResource) iEikonEnv->DeleteResourceFile(iResource);
	delete iContainer;
	delete iErrorInfo;
	delete iSendUi;
}

void CErrorInfoViewImpl::ShowWithData(const MErrorInfo& aInfo)
{
	delete iErrorInfo; iErrorInfo=0;
	iErrorInfo=aInfo.CreateCopyL();
	AppUi()->ActivateLocalViewL(KErrorInfoView);
}
